' The PowerBASIC Wizard's Library

' ArcView - Archive Viewer Demo

' Version 2.20

' Copyright 1991 - 1995 Information Management Systems, Inc.

' This is another simple demo of the PBWiz routines.  It allows you to
' view the files in an archive (.ARC, .ARJ, .LZH, .PAK, .ZIP, or .ZOO).

' Syntax:
'   ARCVIEW arcname [/V]

' arcname is the name of the archive (including the extension) which you
' wish to scan.  The /V parameter (which is optional) allows you to get a
' full listing of the information for each file in the archive.  Without it,
' you will only get a list of the names of the files stored in the archive.


$DIM ARRAY

$INCLUDE "PBWiz.INC"
$LINK "PBWiz.PBL"

' display the name of the program and copyright info to the user
PRINT "ARCVIEW: View Archive Demo for the PB Wizard's Library 2.2"
PRINT "  Copyright 1991 - 1995 Information Management Systems, Inc."
PRINT

' check for /? or no command line parameters; if so, display help
Cmd$ = LTRIM$(RTRIM$(UCASE$(COMMAND$)))
IF INSTR(Cmd$, "/?") OR LEN(Cmd$) = 0 THEN
  PRINT "Syntax:"
  PRINT "  ARCVIEW arcname[.ext] [/V]
  PRINT
  PRINT "Use /V for a full listing, as opposed to just a list of the files contained"
  PRINT "in the archive.  ARCVIEW can currently scan ARC, ARJ, EXE, LZH, PAK, ZIP,"
  PRINT "and ZOO archive file formats."
  END
END IF

' check for a /V on the command line; if there, remove it from Cmd$
tmp% = INSTR(Cmd$, "/V")
IF tmp% THEN
  FullView% = -1
  Cmd$ = LTRIM$(RTRIM$(LEFT$(Cmd$, tmp% - 1) + MID$(Cmd$, tmp% + 2)))
END IF

' check to see if a filename has been specified
IF LEN(Cmd$) THEN
  Arc$ = Cmd$
ELSE
  PRINT "You must specify the name of the archive.  Type ARCVIEW /? for help."
  END
END IF


' determine the type of archive file and display it to the user
Chk% = ArcFindType%(Arc$, ArcType$, ArcType%, 1)	' determine the archive type
IF Chk% THEN	' an error occurred
  SELECT CASE Chk%	' display a message appropriate for the error
    CASE -251
      PRINT "Archive file not found."
    CASE -254
      PRINT "Unknown archive type."
    CASE -255
      PRINT "Invalid or corrupt .EXE or .COM file."
  END SELECT
  END
END IF

PRINT "Archive type: "; ArcType$
PRINT

' print a heading for the display
IF FullView% THEN
  PRINT "File Name     Date        Time   CRC       Orig Size  Curr Size  Method"
  PRINT "------------  ----------  -----  --------  ---------  ---------  --------"
ELSE
  PRINT "File Name"
  PRINT "------------"
END IF

Lines% = 7	' number of lines printed

' scan the archive for all files and display them to the user
Chk% = ArcFindFirst%(Arc$, "*.*", ArcType%, 0)
IF Chk% THEN PRINT "Error on ArcFindFirst%: "; Chk% : END

DO
  IF Lines% = 24 THEN	' the screen is full; allow the user a chance to read the screen before it scrolls
    PRINT "Press any key to continue";
    x$ = INPUT$(1)	' wait for a keystroke
    PRINT		' scroll to the next line
    Lines% = 0		' reset the line counter
  END IF
  IF FullView% THEN	' if doing a full view, get all the info
    ArcGetFileInfo Nam$, Dat$, Tim$, Store$, CRC$, OSize&, CSize&
    PRINT Nam$ TAB(15) Dat$ TAB(27) LEFT$(Tim$, 5) TAB(34) CRC$ TAB(44) _
      USING$("#########", OSize&) TAB(55) USING$("#########", CSize&) _
      TAB(66) Store$
  ELSE	' if not, only get the name (that's all we display, so save time)
    PRINT ArcGetName$
  END IF
  INCR Lines%
  Chk% = ArcFindNext%
LOOP UNTIL Chk%		' loop until an error code is returned (such as no more files)

ArcClose	' close the archive file
END
