' The PowerBASIC Wizard's Library

' EMSDemo - EMS Write/Read Demo

' Version 2.20

' Copyright 1991 - 1995 Information Management Systems, Inc.

' This provides a brief demo of the EMS routines.  It allocates enough
' memory to hold a long integer array of dimensions 10000 x 4 and loads it.
' Note: Due to the large amount of memory that this program manipulates,
' you will probably not be able to run it inside the PowerBASIC IDE.

$DIM ARRAY

$INCLUDE "PBWiz.INC"
$LINK "PBWiz.PBL"


  ' Get general information about the EMS
  CALL EMSVer(VMaj%, VMin%)
  EMSVers$ = RTRIM$(LTRIM$(STR$(VMaj%))) + "." + RTRIM$(LTRIM$(STR$(VMin%)))
  TotFree$ = RTRIM$(LTRIM$(STR$(EMSTotal%)))
  BlocksFree$ = RTRIM$(LTRIM$(STR$(EMSFree%)))


  CLS
  ' display screen title
  Header$ = "EMS Write and Read Demo"
  StrLen% = LEN(Header$)
  Col? = ((80 - StrLen%) \ 2)  ' get the column at which to print the string

  COLOR 3, 0
  LOCATE 1, Col? : PRINT Header$;
  COLOR 7, 0

  ' Make sure EMS is installed.
  IF NOT EMSExists% THEN
    COLOR 15, 4
    ErrMsg$ = "This demo requires EMS memory to run."
    StrLen% = LEN(ErrMsg$)
    Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string
    LOCATE 7, Col? : PRINT ErrMsg$
    COLOR 7, 0
    END
  END IF

  LOCATE 3, 8 : PRINT "EMS Version....: "; EMSVers$;
  LOCATE 4, 8 : PRINT "Total EMS......: "; LTRIM$(USING$("######,###", EMSTotal%)); _
                   " Pages or "; LTRIM$(USING$("######,###", (EMSTotal% * 16384))); _
                   " bytes EMS memory.";
  LOCATE 5, 8 : PRINT "Amount EMS Free: "; LTRIM$(USING$("######,###", EMSFree%)); _
                   " Pages or ";LTRIM$(USING$("######,###", (EMSFree% * 16384))); _
                   " bytes free.";
  COLOR 14, 0
  LOCATE 6, 8 : PRINT STRING$(65, CHR$(196));
  COLOR 7, 0

  ' Set up variables.  We'll be using a 10000 x 4 element array of long
  ' integers in EMS.
  Size1& = 10000 ' elements in first dimension
  Size2& = 4 ' elements in second dimension

  DIM HUGE ABigArr&(1:Size1&, 1:Size2&)
  ArrayBytes& = Size1& * Size2& * 4 ' bytes needed for array
  ArrayKB&  = (ArrayBytes& + 1023&) \ 1024& ' Kbytes needed for array
  ArrayPgs% = (ArrayKB& \ 16) + 1 ' (Kbytes / 16KB per page) needed for array

  IF ArrayPgs% > EMSFree% THEN
    COLOR 15, 4
    ErrMsg$ = "This demo requires approximately 160k of free EMS memory."
    StrLen% = LEN(ErrMsg$)
    Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string
    LOCATE 7, Col? : PRINT ErrMsg$
    COLOR 7, 0
    END
  END IF


  ' Open an area of EMS memory (like DIM for arrays).  If it succeeds, it
  ' will return a value in ArrayName which we'll use to access the opened
  ' memory area.
  CALL EMSOpen(ArrayPgs%, ArrayName%, ErrCode%)
  IF ErrCode% THEN
    COLOR 15, 4
    ErrMsg$ = "Error (" + LTRIM$(STR$(ErrCode%)) + ") allocating EMS.  Unable to proceed."
    StrLen% = LEN(ErrMsg$)
    Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string
    LOCATE 7, Col? : PRINT ErrMsg$
    COLOR 7, 0
    END
  END IF

  LOCATE 7, 1 : PRINT "EMS allocated for "; _
                LTRIM$(USING$("###,###", Size1&)); _
                " x "; LTRIM$(USING$("###,###", Size2&)); _
                " long integer array."
  LOCATE 8, 1 : PRINT "Bytes = "; LTRIM$(USING$("######,###", ArrayBytes&)); _
                " Pages = "; LTRIM$(USING$("######,###", ArrayPgs%));


  ' Load ABigArr& with numbers for entire array space
  LOCATE 10, 1 : PRINT "Filling the array with consecutive numbers."
  cntr& = 0
  FOR row% = 1 TO Size1&
    FOR col% = 1 TO Size2&
      INCR cntr&
      ABigArr&(row%, col%) = cntr&
      LOCATE 11, 2 : PRINT LTRIM$(USING$("######,###", ABigArr&(row%, col%)))
    NEXT col%
  NEXT row%


  ' let user know our array is full & we are ready to copy to EMS
  LOCATE 13, 1 : PRINT "Array has been filled."
  LOCATE 14, 1 : PRINT "Press any key to copy the "; _
                       LTRIM$(USING$("######,###", ArrayBytes&)); _
                      " byte array to EMS."

  ' wait for a keystroke
  x$ = INPUT$(1)

  ' get the varseg & offset of ABigArr&()
  ABigVarSeg& = VARSEG(ABigArr&(1, 1))
  ABigVarOfs& = VARPTR(ABigArr&(1, 1))

  ' Map EMS Virtual Pages, to physical pages,
  ' POKE$ data, then repeat till done
  SectionLength% = ArrayBytes& / ArrayPgs%
  ByteRemains&   = ArrayBytes&

  ' sending up to EMS the array
  LOCATE 15, 1 : PRINT "Copying the array to EMS."

  FOR q% = 1 TO ArrayPgs%
    IF ByteRemains& < SectionLength% THEN SectionLength% = ByteRemains&

    DEF SEG = 0

    CALL EMSMap(ArrayName%, 0, q%)
    EMSSEGTarget& = EMSSeg&

    POKE$(EMSSEGTarget& * 16) + 0, PEEK$((ABigVarSeg& * 16) + ABigVarOfs&, SectionLength%)
    INCR ABigVarOfs&, SectionLength%
    DECR ByteRemains&, SectionLength%

  NEXT q%

  ' tell user EMS write is done
  LOCATE 16, 1
  PRINT "Array has been copied to EMS."

  ' let user know we are ready to read array back from EMS
  LOCATE 17, 1 : PRINT "Press any key to read the "; _
                       LTRIM$(USING$("#####,###", ArrayBytes&)); _
                       " byte array from EMS."

  ' wait for a keystroke
  x$ = INPUT$(1)


  ' Map EMS Virtual Pages, to physical pages,
  ' POKE$ data, then repeat till done
  SectionLength% = ArrayBytes& / ArrayPgs%
  ByteRemains&   = ArrayBytes&

  ' Clear ABigArr& so we can stuff values from EMS back into it.
  REDIM HUGE ABigArr&(1:Size1&, 1:Size2&)

  ' get the varseg & offset of ABigArr&()
  ABigVarSeg& = VARSEG(ABigArr&(1, 1))
  ABigVarOfs& = VARPTR(ABigArr&(1, 1))

  ' Let's read it back from EMS
  LOCATE 18, 1
  PRINT "Copying the array back out of EMS."

  FOR q% = 1 TO ArrayPgs%
    IF ByteRemains& < SectionLength% THEN SectionLength% = ByteRemains&

    DEF SEG = 0

    CALL EMSMap(ArrayName%, 0, q%)
    EMSSEGTarget& = EMSSeg&

    POKE$ ((ABigVarSeg& * 16) + ABigVarOfs&), PEEK$((EMSSEGTarget& * 16) + 0, SectionLength%)
    INCR ABigVarOfs&, SectionLength%
    DECR ByteRemains&, SectionLength%

  NEXT q%

  ' tell user EMS read is done
  LOCATE 19, 1
  PRINT "The array has been copied out of EMS."

  ' reset segment pointer
  DEF SEG

  ' scroll through values by way of verification
  LOCATE 21, 1 : PRINT "Printing array values in order to validate the operation."
  FOR row% = 1 TO 10000
    FOR col% = 1 TO 4
      LOCATE 22, 2 : PRINT LTRIM$(USING$("###,###", ABigArr&(row%, col%)));
    NEXT col%
  NEXT row%

  ' We're all done, so let's return the EMS memory to the system.
  ' This is IMPORTANT, because otherwise the EMS would remain
  ' unavailable until the computer is rebooted.

  ' Close EMS
  CALL EMSClose(ArrayName%)

  LOCATE 24, 1
  PRINT "Done";
