' The Full Power Toolbox

' Sample Routines (Version 3.00)

' Copyright 1991 - 1995 Information Management Systems, Inc.


' A license is hereby granted to the holder to use this source code in any
' program, commercial or otherwise, without receiving the express permission
' of the copyright holder and without paying any royalties, as long as this
' code is not distributed in any compilable format, such as source code
' files, PowerBASIC Unit files, printed listings, etc.


$COMPILE UNIT


' Get Program Name

' This function returns the path and name of the program file currently
' running.  If you store data files with the executable file, you can find
' the directory using this function, or if you need to locate the actual
' executable file or need to know it's name, you can use this function.  The
' path and name of the file are returned as the value of the function.

' Caution: This function requires the GetPSP% function to work properly.


FUNCTION GetProgramName$ LOCAL

  LOCAL PSPAddr%, Offset%, EndPoint%, Temp$

  PSPAddr% = GetPSP%          ' call the GetPSP% function to locate the segment of the PSP
  DEF SEG = PSPAddr%
  DEF SEG = PEEKI(44)         ' location of the environment is at offset 44 in the PSP
  DO                          ' search the environment for the start of the file name
    IF PEEK(Offset%) = 0 THEN        ' the start of the executable file name is marked with 2 nulls
      IF PEEK(Offset% + 1) = 0 THEN
       INCR Offset%, 4
       EXIT LOOP
      END IF
    END IF
    INCR Offset%
  LOOP
  EndPoint% = Offset%
  DO                          ' search for the end of the file name
    IF PEEK(EndPoint%) = 0 THEN EXIT LOOP
    INCR EndPoint%
  LOOP
  Temp$ = PEEK$(Offset%, (EndPoint% - Offset% + 1))
  DEF SEG                     ' restore default data segment

  GetProgramName$ = Temp$

END FUNCTION



' Get PSP Address

' This function returns the segment address of the PSP (Program Segment
' Prefix) of the program that is currently running.


FUNCTION GetPSP% LOCAL PUBLIC

  REG 1, &H62 * 256   ' set AH to 62h
  CALL INTERRUPT &H21 ' call DOS services
  GetPSP% = REG(2)    ' PSP segment is returned in BX

END FUNCTION



' Split Path

' This procedure parses a string that includes a path and file name, such
' as C:\WP\DOCS\LETTERS\JONES01.LTR, splits the path away from the file
' name, and returns the two separate values.

' FilePath$ is the combined path and file name
' Path$ is the path (as returned by the procedure)
' FileName$ is the file name (as returned by the procedure)


SUB SplitPath(FilePath$, Path$, FileName$) LOCAL

  LOCAL StartPos%, c%

  StartPos% = 0
  FOR c% = LEN(FilePath$) TO 1 STEP -1
    IF MID$(FilePath$, c%, 1) = "\" OR MID$(FilePath$, c%, 1) = ":" THEN
      StartPos% = c%
      EXIT FOR
    END IF
  NEXT c%

  FileName$ = RIGHT$(FilePath$, (LEN(FilePath$) - StartPos%))
  Path$ = LEFT$(FilePath$, StartPos%)

END SUB



' Partial Clear Screen

' This procedure clears a rectangular section of the screen.  (It uses BIOS
' for screen writes so should not interfere with windowed environments.)

' ULRow?, ULCol?, LRRow?, and LRCol? are the upper left and lower right
'   row and column coordinates of the area to clear.
' Attr? is the video attribute with which to color the newly cleared area of
'   the screen.


SUB PartialCLS(ULRow?, ULCol?, LRRow?, LRCol?, Attr?) LOCAL PUBLIC

  REG 1, &H0600		' set AH to 6 (BIOS function number) and AL to 0
			' (number of rows to scroll; when set to 0, the area
			' is cleared rather than scrolled)
  REG 2, Attr? * 256	' set BH to the video attribute for the area
  REG 3, (ULRow? * 256) + ULCol?  ' set CH to the upper left row of the
				  ' area and CL to the upper left column of the area
  REG 4, (LRRow? * 256) + LRCol?  ' set DH to the lower right row of the
				  ' area and DL to the lower right column of the area
  CALL INTERRUPT &H10	' call the video BIOS services interrupt

END SUB
