' The PowerBASIC Wizard's Library

' XMSDemo - XMS Write/Read Demo

' Version 2.20

' Copyright 1991 - 1995 Information Management Systems, Inc.

' This provides a brief demo of the XMS routines.  It allocates enough
' memory to hold a long integer array of dimensions 10000 x 4 and loads
' it sequentially.

$DIM ARRAY


$INCLUDE "PBWiz.INC"
$LINK "PBWiz.PBL"


  CLS

  ' Get general information about the XMS
  CALL XMSVer(VMaj%, VMin%)
  XMSVersion$ = RTRIM$(LTRIM$(STR$(VMaj%))) + "." + RTRIM$(LTRIM$(STR$(VMin%)))
  TotFree$ = RTRIM$(LTRIM$(STR$(XMSTFree&)))
  BlockFree$ = RTRIM$(LTRIM$(STR$(XMSLFree&)))


  ' Identify program
  Header$ = "XMS Write and Read Demo"
  StrLen% = LEN(Header$)
  Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string

  ' Make sure XMS is installed.
  IF NOT XMSexists% THEN
    COLOR 15, 4
    ErrMsg$ = "This demo requires XMS memory to run."
    StrLen% = LEN(ErrMsg$)
    Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string
    LOCATE 7, Col? : PRINT ErrMsg$
    COLOR 7, 0
    END
  END IF

  COLOR 3, 0
  LOCATE 1, Col? : PRINT Header$;
  COLOR 7, 0

  Col? = ((80 - 65) \ 2)
  LOCATE 3, Col? : PRINT "XMS Version...........: "; XMSVersion$;
  LOCATE 4, Col? : PRINT "Total XMS Free........: "; LTRIM$(USING$("####,###", XMSTFree&)); " KB";
  LOCATE 5, Col? : PRINT "Largest free XMS block: "; LTRIM$(USING$("####,###", XMSLFree&)); " KB";

  COLOR 14, 0
  LOCATE 6, Col? : PRINT STRING$(65, CHR$(196));
  COLOR 7, 0


  ' Set up variables.  We'll be using a 10000x4 element array of
  ' long integers in XMS.
  Size1& = 10000 ' elements in first dimension
  Size2& = 4 ' elements in second dimension

  DIM HUGE ABigArr&(1:Size1&, 1:Size2&)
  ArrayBytes& = Size1& * Size2& * 4 ' bytes needed for array
  ArrayKB& = (ArrayBytes& + 1023&) \ 1024& ' Kbytes needed for array

  IF ArrayKB& > XMSLFree& THEN
    COLOR 15, 4
    ErrMsg$ = "This demo requires approximately 160k of free XMS memory."
    StrLen% = LEN(ErrMsg$)
    Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string
    LOCATE 7, Col? : PRINT ErrMsg$
    COLOR 7, 0
    END
  END IF


  ' Open an area of XMS memory (like DIM for arrays).
  ' If it succeeds, it will return a value in ArrayName
  ' which we'll use to access the opened memory area.
  CALL XMSopen(ArrayKB&, ArrayName%, ErrCode%)
  IF ErrCode% THEN
    COLOR 15, 4
    ErrMsg$ = "Error allocating XMS.  Unable to proceed."
    StrLen% = LEN(ErrMsg$)
    Col? = ((80 - StrLen%) \ 2)  ' get the column on which to print the string
    LOCATE 7, Col? : PRINT ErrMsg$
    COLOR 7, 0
    END
  END IF

  LOCATE 7, 1 : PRINT "XMS allocated for "; _
                LTRIM$(USING$("######,###", Size1&)); " x "; _
                LTRIM$(USING$("######,###", Size2&)); _
                " long integer array.  Size = "; _
                LTRIM$(USING$("#######,###", ArrayBytes&)); " bytes."


  ' Load ABigArr& with numbers for entire array space
  PRINT "Filling the array with consecutive numbers."
  cntr& = 0
  FOR row% = 1 TO 10000
    FOR col% = 1 TO 4
      INCR cntr&
      ABigArr&(row%, col%) = cntr&
      LOCATE 9, 2 : PRINT LTRIM$(USING$("######,###", ABigArr&(row%, col%)));
    NEXT col%
  NEXT row%


  ' let user know our array is full & we are ready to copy to XMS
  LOCATE 11, 1 : PRINT "Array has been filled."
  LOCATE 12, 1 : PRINT "Press any key to copy the "; _
                       LTRIM$(USING$("#####,###", ArrayBytes&)); _
                       " byte array to XMS."

  ' wait for a keystroke
  x$ = INPUT$(1)

  ' get the varseg & offset of ABigArr&()
  ABigVarSeg% = VARSEG(ABigArr&(1, 1))
  ABigVarOfs% = VARPTR(ABigArr&(1, 1))

  ' sending up to XMS the array
  LOCATE 14, 1 : PRINT "Copying the array to XMS."

  CALL XMSwrite(ArrayName%, 0, ArrayBytes&, ABigVarSeg%, ABigVarOfs%)

  ' tell user XMS write is done
  LOCATE 15, 1
  PRINT "Array has been copied to XMS."

  ' let user know we are ready to read array back from XMS
  LOCATE 17, 1 : PRINT "Press any key to read the "; _
                       LTRIM$(USING$("#####,###", ArrayBytes&)); _
                       " byte array from XMS."

  ' wait for a keystroke
  x$ = INPUT$(1)

  ' clear array out for reuse
  REDIM HUGE ABigArr&(1:Size1&, 1:Size2&)

  ' get the varseg & offset of ABigArr&()
  ABigVarSeg% = VARSEG(ABigArr&(1, 1))
  ABigVarOfs% = VARPTR(ABigArr&(1, 1))


  ' Let's read it back from XMS
  LOCATE 19, 1
  PRINT "Copying the array back out of XMS."
  CALL XMSread(ArrayName%, 0, ArrayBytes&, ABigVarSeg%, ABigVarOfs%)

  ' tell user XMS read is done
  LOCATE 20, 1
  PRINT "The array has been copied out of XMS."

  ' scroll through values by way of verification
  LOCATE 22, 1 : PRINT "Printing array values in order to validate the operation."
  FOR row% = 1 TO 10000
    FOR col% = 1 TO 4
      LOCATE 23, 2 : PRINT LTRIM$(USING$("###,###", ABigArr&(row%, col%)));
    NEXT col%
  NEXT row%

  ' We're all done, so let's return the XMS memory to the system.
  ' This is IMPORTANT, because otherwise the XMS would remain
  ' unavailable until the computer is rebooted.

  XMSclose ArrayName%

  LOCATE 24, 1
  PRINT "Done";
